/*
 * gsm_cli.h
 *
 *  Created on: Oct 24, 2018
 *      Author: Vladimir Sustek
 */

#ifndef GSM_DRIVER_H_
#define GSM_DRIVER_H_

#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <usart.h>
#include <main.h>
#include <tim.h>
#include <iwdg.h>

/******************* (1.0) Timer related constants *******************/
#define SECONDS_5 			(uint32_t)(5000u)
#define SECONDS_2 			(uint32_t)(2000u)
#define SECOND_0_5 			(uint32_t)(500u)

#define GET_TIMER_STATE 	(uint32_t)(0u)
#define START_THE_TIMER 	(uint32_t)(1u)
#define RESET_THE_TIMER 	(uint32_t)(2u)
#define TIMER_IS_RUNNING 	(uint32_t)(3u)

/************ (2.0) Visualize LED related constants ******************/
#define NO_FLASH 			(uint32_t)(0u)
#define NO_FLASH 			(uint32_t)(0u)
#define ONCE_FLASH 			(uint32_t)(1u)
#define TWICE_FLASH			(uint32_t)(2u)
#define FAST_FLASH 			(uint32_t)(3u)

/************ (3.0) UART related constants and MACRO *****************/

#define UART_BUFFSIZE		(uint32_t)(512u)
#define MMS_MAX_BUFF 		(uint32_t)(16384)
#define MMS_USR_BUFF 		MMS_MAX_BUFF - 4
#define CLEAR_UART_ISR 		(uint32_t)(1u)
#define GET_UART_ISR 		(uint32_t)(0u)
#define UART_TO 			(uint32_t)(100)

/************ (4.0) Expected AT responses of the M95 *****************/
#define RSPS_OK				(char*)(at_response_list[0])
#define RSPS_CREG 			(char*)(at_response_list[1])
#define RSPS_CGREG 			(char*)(at_response_list[2])
#define RSPS_RING 			(char*)(at_response_list[3])
#define RSPS_SM 			(char*)(at_response_list[4])
#define RSPS_CLLRDY 		(char*)(at_response_list[5])
#define RSPS_GGMNTF 		(char*)(at_response_list[6])
#define RSPS_SMSRD			(char*)(at_response_list[7])
#define RSPS_NOCRR			(char*)(at_response_list[8])
#define RSPS_CMEERR			(char*)(at_response_list[9])
#define RSPS_ERR			(char*)(at_response_list[10])
#define RSPS_UPL			(char*)(at_response_list[11])
#define RSPS_CONNECT		(char*)(at_response_list[12])
#define RSPS_DEACTOK		(char*)(at_response_list[13])
#define RSPS_CMSERR			(char*)(at_response_list[14])
/* 10 - base number representation - used during parsing*/
#define CONVERT_10_BASE (uint32_t)(10)

/*********** (5.0) List of constant AT commands of the M95 ***********/
#define GSM_QDEL_DATA_TXT	(char*)(atok_command_list[6])
#define GSM_QMMSCS_QMM_0	(char*)(atok_command_list[7])
#define GSM_QMMSSW_QMM_1	(char*)(atok_command_list[8])
#define GSM_QMMSCS_UTF8		(char*)(atok_command_list[9])
#define GSM_QMMSW_QMM_4		(char*)(atok_command_list[10])
#define GSM_QMMSW_QMM_5		(char*)(atok_command_list[11])
#define GSM_QMMSEND			(char*)(atok_command_list[12])
#define GSM_QFUPL			(char*)(atok_command_list[13])
#define GSM_QIDEACT			(char*)(atok_command_list[14])
#define GSM_AT				(char*)(atok_command_list[15])

/************* (6.0) Limit constants for initialization **************/
#define GSM_INIT 			(uint32_t)(6)
#define SMS_MAX_CNT			(uint32_t)(21)

/******* (7.0) Constants for response parsing representation *********/
#define GSM_NULL			(int32_t)(0)
#define GSM_NO_ACT			(int32_t)(0)
#define GSM_CMSERR			(int32_t)(-3)
#define GSM_CMEERR			(int32_t)(-2)
#define GSM_ERR 			(int32_t)(-1)
#define GSM_OK 				(int32_t)(1)
#define GSM_RING 			(int32_t)(2)
#define GSM_CLLRDY 			(int32_t)(3)
#define GSM_CREGOK			(int32_t)(4)
#define GSM_CGREGOK			(int32_t)(5)
#define GSM_MMS_REQ			(int32_t)(6)
#define GSM_MMS_SENT		(int32_t)(1000)
#define GSM_SMS_READ		(int32_t)(8)
#define GSM_SMS_DEL			(int32_t)(9)
#define GSM_SMS_RCVD 		(int32_t)(10)
#define GSM_MMS_PENDING		(int32_t)(36)
#define GSM_MMS_TO_MAX		(int32_t)(64)
#define GSM_MMS_POST		(int32_t)(38)
#define GSM_MMS_QFUPL		(int32_t)(39)
#define GSM_CONNECT			(int32_t)(40)
#define GSM_DEACTOK			(int32_t)(41)
#define GSM_DATAUPLD		(int32_t)(42)
#define GSM_ACK_ENABLE		(int32_t)(43)
#define GSM_ACK_DISABLE		(int32_t)(44)
#define GSM_NOCRR			(int32_t)(45)

/* Tokeninizing control constants*/
#define BREAK_AT_GSM_SMS_READ 		(int32_t)(1)
#define DONT_BREAK_AT_GSM_SMS_READ 	(int32_t)(0)

/************* (7.0)  GSM module representation type  ****************/
typedef struct gsm_device_type {
	int32_t cmd_val; /* value of parsed response - see 4.0 and 7.0 of this file*/
	int32_t tim_reqd; /* flag of timer request yes/no */
	int32_t rst_req; /* flag of reset request yes/no */
	int32_t sms_num; /* received SMS number in memory */
	int32_t upload_mode; /* variable for MMS upload state distinguish - value */
	int32_t rst_beep; /* variable for counting dialed beep - tones for system reset */
	uint32_t uart_attempt; /* variable for counting missed UART communication */
	char cmd_to_send[UART_BUFFSIZE]; /* array for customized AT command to be sent */
	char contact[UART_BUFFSIZE/2]; /* array for MMS recipient - email address */
	char *mms_pld; /* pointer on the MMS content to be sent */
	char *p_rspd; /* pointer on the received response from the GSM module*/
} gsm_device_t;

/************* (8.0)  UART3 flag of the received <LF> ****************/
extern volatile int uart3_received_flag;

/************* (9.0) The constant arrays *****************************/
extern const char atok_command_list[16][50]; /* constant commands - linked with 5.0 */
extern const char atreq_command_list[3][16]; /* constant requests */
extern const char at_response_list[15][16]; /* constant responses - linked with 4.0 */

/************ (9.0) Functions for PIN control ************************/
extern int gsm_pwr_supply_pull(int state);
extern int gsm_pwrkey_pull(int state);
extern int gsm_nrts_pull(int state);
extern int green_led_mode_ctl(int mode);
extern int blue_led_mode_ctl(int mode);
extern void green_led_ctl(int mode);
extern void blue_led_ctl(int mode);

/************ (9.0) Functions of the UART control *********************/
extern int gsm_init_ring_DMA_UART(void);
extern int gsm_stop_MMS_DMA_UART(gsm_device_t *module);
extern int gsm_start_MMS_DMA_UART(gsm_device_t *module);
extern int refresh_uart1(gsm_device_t* module);
extern int refresh_uart3(gsm_device_t* module);
extern int uart1_restore_buffer(void);
extern char* uart3_store_received(char recvd, gsm_device_t* module);
extern void gsm_uart_resolve_bugs(void);
extern int uart3_publish_rx_pointer(void);
extern int uart3_force_rx_pointer(int value);

/************ (9.0) UART based TX-RX functions ************************/
extern int gsm_at_cmd_send(char *cmd, int appd_crlf);
extern int gsm_at_data_send(char *data, int lng);
extern void gsm_send_at_SUB(void);
extern int gsm_send_cmd_ack_req2(const char *cmd, gsm_device_t *module);
int gsm_send_cmd_ack_req3(const char *cmd, gsm_device_t *module);
int _write(int file, char const *buf, int n);
extern int poteron_UART_tx(const char *str);

/************ (10.0) Initialization routine **************************/
extern int gsm_module_sys_init(gsm_device_t *module);
extern void gsm_pwr_module_init(void);
extern int gsm_init_module_struct(gsm_device_t* module);

/************ (11.0) String processing commands **********************/
extern int gsm_process_respond_fsm(char* str, int break_at_GSM_SMS_READ);
extern int gsm_send_txt_mms(gsm_device_t *module);
extern int gsm_get_email_recipient(char *sms_content, gsm_device_t *module);
extern char* gsm_read_sms_content(char * str);

/*********************** (12.0) Timers *******************************/
void timer_2_second(void);
void timer_500miliseconds(void);
extern int tim_gsm_respnd(int arg);
extern int tim_cmd_limit(int arg);

#endif /* GSM_DRIVER_H_ */
