/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 ** This notice applies to any and all portions of this file
 * that are not between comment pairs USER CODE BEGIN and
 * USER CODE END. Other portions of this file, whether
 * inserted by the user or by software development tools
 * are owned by their respective copyright owners.
 *
 * COPYRIGHT(c) 2018 STMicroelectronics
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of STMicroelectronics nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f3xx_hal.h"
#include "dma.h"
#include "iwdg.h"
#include "spi.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* USER CODE BEGIN Includes */
/*!
 * @file      main.c
 * @brief     GSM Re-Transmitter based on the Mikroe GSM 2 (Quectel M95) and STM32F373RCT6.
 *            System receives SMS, clears its header and bottom and publish it, part of the
 *            awaited SMS content is email address - it is obtained during SMS routines.
 *            If any data are received via UART1 (more than 10 characters) the MMS sending
 *            routine is started to send the data via MMS to the default or obtained email
 *            address, note that recipient may be phone number, but only Czech numbers are
 *            obtained from the received SMS. Reaction to the module dialing is complete
 *            reset if is module dialed for more than 4 - dialing tones. There are used two
 *            signals LEDs: Green_LED, Blue_LED. The green LED is flashing constantly during
 *            the initialization, flashes once per second during normal operational mode, and
 *            flashes twice a second if GSM interfacing is currently lost. The Blue_LED shows
 *            MMS receive routines (flashes once per second) and its usage may be extended.
 *
 * @details
 *            The system is at the first initialized using functions between USER CODE BEGIN 2
 *            and USER CODE END 2. And then starts FSM (USER CODE BEGIN WHILE), whereas is
 *            response from the GSM module processed, by default response is set to GSM_OK to
 *            set another empty AT command (The most of the time are only sent empty AT commands
 *            - the way how to get current state of the module). After the command is sent, the
 *            timer runs for two seconds, before response is processed using the GSM MODULE
 *            RECEIVING SECTION. If the time elapsed, the counter of missed cycles is incremented
 *            in case of a few missed attempts the UART3, which is interfacing the module, is
 *            restarted and again sent empty AT command using GSM_OK cmd_val settings. In case of
 *            any problem (inappropriate sent MMS or module dialed) is system reseted triggered by
 *            rst_req flag or 26 second WatchDog. The infinite loop ends at USER CODE END WHILE.
 *
 */
#include <string.h>
#include "gsm_driver.h"
#include "gsm_dal.h"
/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
 * @brief  The application entry point.
 *
 * @retval None
 */
int main(void) {
	/* USER CODE BEGIN 1 */

	/* Structure representing GSM module including flags, recipient, response .. */
	gsm_device_t gsm_module;
	gsm_device_t *gsmmod = &gsm_module;
	int init_result = 0;

	/* USER CODE END 1 */

	/* MCU Configuration----------------------------------------------------------*/

	/* Reset of all peripherals, Initializes the Flash interface and the Systick. */
	HAL_Init();

	/* USER CODE BEGIN Init */

	/* USER CODE END Init */

	/* Configure the system clock */
	SystemClock_Config();

	/* USER CODE BEGIN SysInit */

	/* USER CODE END SysInit */

	/* Initialize all configured peripherals */
	MX_GPIO_Init();
	MX_DMA_Init();
	MX_TIM2_Init();
	MX_USART3_UART_Init();
	MX_USART1_UART_Init();
	MX_USART2_UART_Init();
	MX_IWDG_Init();
	MX_SPI1_Init();
	/* USER CODE BEGIN 2 */

	HAL_TIM_Base_Start_IT(&htim2);
	/* GSM module platform initialization*/
	green_led_mode_ctl(FAST_FLASH);
	blue_led_mode_ctl(NO_FLASH);
	gsm_init_module_struct(gsmmod);
	gsm_pwr_module_init();
	CLEAR_WATCH_DOG();
	gsm_init_ring_DMA_UART();
	init_result = gsm_module_sys_init(gsmmod);
	printf("__INITIALIZATION COMPLETED WITH RESULT: %d__\r\n", init_result);
	if (init_result < GSM_NO_ACT) {
		gsmmod->rst_req = 1;
		gsmmod->cmd_val = GSM_ERR;
	}

	CLEAR_WATCH_DOG();
	gsm_start_MMS_DMA_UART(gsmmod);
	/* USER CODE END 2 */

	/* Infinite loop */
	/* USER CODE BEGIN WHILE */
	while (1) {

		/********** GSM MODULE STATE MACHINE, PROCESSING SECTION - SOLVE THE STATE***********/

		switch (gsmmod->cmd_val) {

		case GSM_OK :
			gsm_idle_state(gsmmod);
			break;
		case GSM_SMS_RCVD :
			gsm_SMS_tbread(gsmmod);
			break;
		case GSM_SMS_READ :
			gsm_SMS_publish(gsmmod);
			break;
		case GSM_SMS_DEL :
			gsm_SMS_del(gsmmod);
			break;
		case GSM_RING :
			gsm_mod_dialed(gsmmod);
			break;
		case GSM_MMS_POST :
			gms_MMS_send(gsmmod);
			break;
		case GSM_NOCRR :
			gsm_mod_hangup(gsmmod);
			break;
		default: /* Nothing to do */
			break;
		}

		/* Start the timer */
		if (gsmmod->tim_reqd) {
			tim_gsm_respnd(START_THE_TIMER);
		}

		/******************** GSM MODULE RECEIVING SECTION - GET STATE-**********************/

		/* Check the UART DMA ring buffers if anything received*/
		refresh_uart3(gsmmod);
		refresh_uart1(gsmmod);

		/* Command had been sent and module responded + response time elapsed*/
		if (uart3_received_flag && tim_gsm_respnd(GET_TIMER_STATE)) {

			/* Process received response, reset timers */
			gsm_rec_respnd2(gsmmod, 1);
			tim_gsm_respnd(RESET_THE_TIMER);
			green_led_mode_ctl(ONCE_FLASH);
			uart3_received_flag = 0;
			gsmmod->uart_attempt = 0u;
			CLEAR_WATCH_DOG();
		}
		/* Module does not respond - give another chance eventually reset TX and RX GSM UART */
		else if (tim_gsm_respnd(GET_TIMER_STATE)) {
			gsm_missed_response(gsmmod);
		}
		/* Uploading data by the UART1 from the measurement device to sent it as MMS*/
		gsm_upload_mode(gsmmod);

		/******************** USER REMOTE RESET ROUTINE *************************************/

		if (gsmmod->rst_req) {
			/* Full system reset including Poteron Reset routine - all data will be lost*/
			printf("__COMPLETE_RESET__\r\n");
			NVIC_SystemReset();
		}

		/* USER CODE END WHILE */

		/* USER CODE BEGIN 3 */
	}
	/* USER CODE END 3 */

}

/**
 * @brief System Clock Configuration
 * @retval None
 */
void SystemClock_Config(void) {

	RCC_OscInitTypeDef RCC_OscInitStruct;
	RCC_ClkInitTypeDef RCC_ClkInitStruct;
	RCC_PeriphCLKInitTypeDef PeriphClkInit;

	/**Initializes the CPU, AHB and APB busses clocks
	 */
	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI
			| RCC_OSCILLATORTYPE_LSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = 16;
	RCC_OscInitStruct.LSIState = RCC_LSI_ON;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK) {
		_Error_Handler(__FILE__, __LINE__);
	}

	/**Initializes the CPU, AHB and APB busses clocks
	 */
	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK
			| RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK) {
		_Error_Handler(__FILE__, __LINE__);
	}

	PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1
			| RCC_PERIPHCLK_USART2 | RCC_PERIPHCLK_USART3;
	PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK2;
	PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_PCLK1;
	PeriphClkInit.Usart3ClockSelection = RCC_USART3CLKSOURCE_PCLK1;
	if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK) {
		_Error_Handler(__FILE__, __LINE__);
	}

	/**Configure the Systick interrupt time
	 */
	HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq() / 1000);

	/**Configure the Systick
	 */
	HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

	/* SysTick_IRQn interrupt configuration */
	HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
 * @brief  This function is executed in case of error occurrence.
 * @param  file: The file name as string.
 * @param  line: The line in file as a number.
 * @retval None
 */
void _Error_Handler(char *file, int line) {
	/* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */
	while (1) {
	}
	/* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t* file, uint32_t line)
{
	/* USER CODE BEGIN 6 */
	/* User can add his own implementation to report the file name and line number,
	 tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
	/* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/**
 * @}
 */

/**
 * @}
 */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
